@@ -787,7 +787,7 @@ def file_fetch(fh, fromTime, untilTime, now = None):
   if untilTime > now:
     untilTime = now
 
-  diff = now - fromTime
+  diff = untilTime - fromTime
   for archive in header['archives']:
     if archive['retention'] >= diff:
       break
@@ -802,6 +802,9 @@ archive on a read and request data older than the archive's retention
 """
   fromInterval = int( fromTime - (fromTime % archive['secondsPerPoint']) ) + archive['secondsPerPoint']
   untilInterval = int( untilTime - (untilTime % archive['secondsPerPoint']) ) + archive['secondsPerPoint']
+  if fromInterval == untilInterval:
+    # Zero-length time range: always include the next point
+    untilInterval += archive['secondsPerPoint']
   fh.seek(archive['offset'])
   packedPoint = fh.read(pointSize)
   (baseInterval,baseValue) = struct.unpack(pointFormat,packedPoint)
